@@ -2,8 +2,6 @@ from .action import NHCAction
 from .const import PRESET_MODES, MODES
 
 class NHCFan(NHCAction):
-  def __init__(self, controller, action):
-    super().__init__(controller, action)
 
   @property
   def modes(self) -> list:
@@ -17,5 +15,5 @@ class NHCFan(NHCAction):
 
     return PRESET_MODES['low']
 
-  def set_mode(self, preset_mode: str) -> None:
-      self._controller.execute(self.id, PRESET_MODES[preset_mode])
+  async def set_mode(self, preset_mode: str) -> None:
+      await self._controller.execute(self.id, PRESET_MODES[preset_mode])
