@@ -3,11 +3,19 @@ from .action import NHCBaseAction
 class NHCThermostat(NHCBaseAction):
     def __init__(self, controller, data):
         super().__init__(controller, data)
-        self._measured = data["measured"]
-        self._setpoint = data["setpoint"]
+        self._measured = data["measured"] / 10
+        self._setpoint = data["setpoint"] / 10
         self._overrule = data["overrule"]
         self._overruletime = data["overruletime"]
         self._ecosave = data["ecosave"]
+
+    @property
+    def id(self):
+        return f"thermostat-{self._id}"
+    
+    @property
+    def action_id(self):
+        return self._id
     
     @property
     def measured(self):
@@ -33,16 +41,16 @@ class NHCThermostat(NHCBaseAction):
     def ecosave(self):
         return self._ecosave
     
-    def set_mode(self, mode):
-        self._controller.execute_thermostat(self.id, mode, self._overruletime, self._overrule, self._wanted)
+    async def set_mode(self, mode):
+        await self._controller.execute_thermostat_mode(self._id, mode, self._overruletime, self._overrule)
 
-    def set_temperature(self, setpoint):
-        self._controller.execute_thermostat(self.id, self._state, self._overruletime, self._overrule, setpoint)
+    async def set_temperature(self, setpoint):
+        await self._controller.execute_thermostat_set_temperature(self._id, setpoint * 10)
     
     def update_state(self, data):
         self._state = data["mode"]
-        self._setpoint = data["setpoint"]
-        self._measured = data["measured"]
+        self._setpoint = data["setpoint"] / 10
+        self._measured = data["measured"] / 10
         self._overrule = data["overrule"]
         self._overruletime = data["overruletime"]
         self._ecosave = data["ecosave"]
